% SOR Successive Over-Relaxation Method
%
% function [x,flag,relres,iter,resids]   = sor(A, b, tol, maxit, varargin) 
%
%   This function solves linear equation systemsuch as A*x=b using the SOR 
%   method (Successive Over-Relaxation).
%   When the relaxation scalar omega=1, the method used is Gauss-Seidel.

function [x,flag,relres,iter,resids]   = sor(A, b, tol, maxit, varargin) 

    omega = 1.4;
    if (nargin >= 5)
        omega =varargin {1};
    end
    
    flag = 0;% zero indicates success
    iter = 0;% no iterations yet

    normb2 = norm(b,2);
    if (normb2 == 0.0) % zero right-hand side, nothing to compute
        x=0*b;
        return
    end
    
    x=0*b; % initial guess

    r = b - A * x;
    relres = norm(r,2) / normb2;
    resids(1)=relres;
    
    if (resids(end) <= tol)
        return
    end

    % Prepare matrices for SOR/Gauss-Seidel
    n =size(A,1);
    sb = omega * b;
    D = spdiags(spdiags(A,0),0,n,n);
    M =  omega * tril(A,-1) + D;
    N = -omega * triu(A,1) + (1.0 - omega) * D;

    for iter = 1 : maxit
        px = x;
        x = M \ (N * x + sb); 
        r = b - A * x;
        relres = norm(r,2) / normb2;
        resids(end+1)=relres;
        if (resids(end) <= tol) % check for convergence
            return
        end          
    end
    % No convergence
     flag = 1;
end
