% Measure a function over the discrete manifold. 
%
% function result = measure (self, geom, fh, varargin)
%
% Integrate some scalar function over the geometric cells. When the scalar
% function returns just +1 [measure(feb,geom,inline('1'))], the result
% measures the volume (number of points, length, area, 3-D volume,
% according to the manifold dimension). When the function returns the mass
% density, the method measures the mass, when the function returns the
% x-coordinate equal measure the static moment with respect to the y- axis,
% and so on.  
% 
% Arguments: self = feblock, geom = geometry field, fh   = function handle
% varargin= optional: manifold dimension to be supplied to Jacobian_mdim().
% 
% Example:
%     V=measure(feb,geom,@(x)(1))% Volume
%     S=measure(feb,geom,@(x)(x))% Static moments
%     CG=S/V% Center of gravity
%     % Now compute the moments of inertia
%     I=measure(feb,geom,@(x)(norm(x-CG)^2*eye(3)-(x-CG)'*(x-CG)))
%     mass=V*rhos;
%     Inertia=I*rhos;
function result = measure (self, geom, fh, varargin)
    gcells =self.gcells;
    if nargin >=4
        m =varargin{1};% Either the manifold dimension was supplied
    else
        m= get(gcells(1),'dim');% ...Or it is implied
    end
    ngcells = count(self.gcells);
    % Integration rule
    integration_rule = get(self, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Precompute basis f. values + basis f. gradients wrt parametric coor
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    result = 0;% Initialize the result
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    for i=1:ngcells
        conn =conns(i,:);
        x = gather(geom, conn, 'values', 'noreshape');
        % Loop over all integration points
        for j=1:npts_per_gcell
            J = Jacobian_matrix (gcells, Nders{j}, x);
            Jac =Jacobian_mdim(gcells, conn, Ns{j}, J, x, m);
            result = result + fh(Ns{j}'*x)*Jac*w(j);
        end
    end
end
