% Compute the acoustic ABC (Absorbing Boundary Condition) 
% matrices of the individual gcells.
%
% function ems = surface_acoustic_ABC(self, geom, dP)
%
% Return an array of the element matrices so they may be assembled.
%    Arguments
%     geom=geometry field
%     dP=time derivative of the acoustic pressure field

function ems = surface_acoustic_ABC(self, geom, dP)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Precompute basis f. values + basis f. gradients wrt parametric coor
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material
    mat = get(self.feblock, 'mater');
    bulk = get(get(mat,'property'),'bulk');
    rho = get(get(mat,'property'),'rho');
    c = sqrt(bulk/rho);% sound speed
    % Prepare some data: connectivity and the element matrices
    conns = get(gcells, 'conn'); % connectivity
    Ke = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(dP,conns(i,:),'eqnums');
        Ke{i} =zeros(nfens);
    end
    xs =get(geom,'values');
    for i=1:size(conns,1)
        conn =conns(i,:);
        x=xs(conn,:);
        for j=1:npts_per_gcell
            J = Jacobian_matrix(gcells,Nders{j},x);
            Jac = Jacobian_surface(gcells,conn, Ns{j}, J, x);
            Ke{i} = Ke{i} + ((Jac/c*w(j))*Ns{j})*Ns{j}';
        end
    end
    ems  = elematset(struct('mat',{Ke}, 'eqnums',{eqnums}));
end
