% Inspect the integration point quantities.
% by which the individual gcells act on the nodes.
% Return an array of elevec objects so they may be assembled.
% Input arguments
%    x=reference geometry field
%    u=displacement field
%    dT=temperature difference field
%     context    - structure: see the update() method.
%     gcell_list - indexes of the geometric cells that are to be inspected
%     inspector - function handle or in-line function with the signature
%             inspector_data =inspector(inspector_data, out, xyz),
%        where
%     inspector_data - a structure or an array that the inspector may
%                use to maintain some state, for instance minimum or
%                maximum of stress, out is the output  of the update()
%                method, and xyz is the location of the integration point
%                in the *reference* configuration.
% Output arguments
%     inspector_data - see the description of the input argument
%
function inspector_data = inspect_integration_points(self, geom, u, dT, gcell_list, context, inspector, inspector_data)
gcells = get(self.feblock,'gcells');
ngcells = count(gcells);
nfens = get(gcells(1),'nfens');
dim = get(geom,'dim');
% Integration rule
integration_rule = get(self.feblock, 'integration_rule');
pc = get(integration_rule, 'param_coords');
w  = get(integration_rule, 'weights');
npts_per_gcell = length(pc); % number of integration point
% Material
mat = get(self.feblock, 'mater');
matstates = get(self, 'matstates');
context.F= [];
% Now loop over all gcells in the block
conns = get(gcells, 'conn'); % connectivity
for m=1:length(gcell_list)
    i=gcell_list(m);
    conn=conns(i,:);
    
    X = gather(geom, conn, 'values', 'noreshape'); % reference coordinates
    U = gather(u, conn, 'values', 'noreshape'); % displacement
    if isempty(dT)
        dTs=zeros(nfens,1);
    else
        dTs = gather(context.dT, conn, 'values', 'noreshape'); % displacement
    end
    % Loop over all integration points
    for j=1:npts_per_gcell

        Nder = bfundpar (gcells, pc(j,:));
        NderX = bfundsp (gcells, Nder, X);
        context.F = (X+U)'*NderX;
        N = bfun(gcells, pc(j,:));
        context.dT = dTs'*N;
        [out,matstates{i,j}] = update(mat, matstates{i,j}, context);
        xyz = N'*X;% Note: evaluated in the reference geometry
        if ~isempty (inspector)
            inspector_data =feval(inspector,inspector_data,out,xyz);
        end
    end
end
return;
