% Compute the restoring force vectors
% by which the individual gcells act on the nodes.
% Return an array of elevec objects so they may be assembled.
%    Call as
% evs = restoring_force(feb, geom, u1, u), or
% [feb,evs] = restoring_force(feb, geom, u1, u)
%     geom     - geometry field
%     u1       - displacement field at the end of time step t_n+1
%     u        - displacement field at the end of time step t_n
%
% Note: This method *UPDATES* the state of the feblock object.  In
%       particular, the material state gets updated.  If this gets
%       called for converged u1, the feblock must be assigned to itself
%       on return; otherwise the first return value must be ignored!
%
function [self,evs] = restoring_force(self, geom, u1, u)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells,'nfens');
    dim = get(geom,'dim');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = length(pc); % number of integration point
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    % Material
    mat = get(self.feblock, 'mater');
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Fe{i} =zeros(get(geom,'dim')*nfens,1);
    end
    Xs = gather(geom, (1:get(geom,'nfens')), 'values', 'noreshape'); % reference coordinates
    Us = gather(u, (1:get(geom,'nfens')), 'values', 'noreshape'); % displacement in step n
    U1s = gather(u1, (1:get(geom,'nfens')), 'values', 'noreshape'); % displacement in step n+1
    matstates = self.matstates;
    for i=1:size(conns,1)
        conn =conns(i,:); % connectivity
        X=Xs(conn,:);
        U=Us(conn,:);
        U1=U1s(conn,:);
        x = X + U; % previous known coordinates
        x1 = X + U1; % current coordinates
        context.dT = 0;
        for j=1:npts_per_gcell
            c =Ns{j}'*X;% physical location of the quadrature point
            J = x1' * Nders{j};% We compute the Jacobian matrix
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,J);
            end
            if (isempty(Rm))
                Nderx1 = Nders{j}/J;% derivatives wrt global coor
                NderX=Nders{j}/(X' * Nders{j});
            else
                Nderx1 = Nders{j}/(Rm'*J);% derivatives wrt local coor
                NderX=Nders{j}/(Rm'*X' * Nders{j});
            end
            Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x1);
            if (Jac<=0),error('Non-positive Jacobian');end
            F1 = x1'*NderX;%def_grad(gcells(i), NderX, x1);
            [cauchy,matstates{i,j}] = update(mat, matstates{i,j}, struct ('output','Cauchy','F',F1));
            B = self.hBlmat(self,Ns{j},Nderx1,c,Rm);% strain-displacement
            Fe{i} = Fe{i} - B'* (cauchy * Jac * w(j)); % note the sign
        end
    end
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
    self.matstates = matstates;
    return;
end