% Constructor of the finite element block class to be used with 
% continuum elements.
%    This class implements large displacement, large strain mechanics.
%    for continuum elements with nodal integration (NICE).
% Reference: Assumed-deformation gradient finite elements with nodal integration 
%for nearly incompressible large deformation analysis,
%M. Broccardo, M. Micheloni, P. Krysl,*
%International Journal for Numerical Methods in Engineering,
%Volume 78, Issue 9, pages 11131134, 28 May 2009, 
%http://onlinelibrary.wiley.com/doi/10.1002/nme.2521/abstract
%
% Always takes either zero or one argument.  For no arguments, the default
% object is created.  For one argument, the argument may be either an
% object of the same type, in which case the copy is created; or, it may be
% a struct with the following fields representing options.
% Options: those recognized by feblock
%
function retobj = feblock_defor_nonlinear_nice (varargin)
    class_name='feblock_defor_nonlinear_nice';
    if (nargin == 0)
        parent = feblock;
        self.matstates = [];
        self.hBlmat=[];%%%%%
        self.hGmat=[];%%%%%
        self.Rm= [];
        self.N = [];
        self.Nder= [];
%         self.overlay= false;
        retobj = class(self,class_name,parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            options =varargin{1};
            parent = feblock(varargin{:});
            self.matstates = [];
            self.hBlmat=[];%%%%%
            self.hGmat=[];%%%%%
            self.hadjugate=[];
            gcells = get(parent,'gcells');
            if (count (gcells) < 1)
                error('Need at least one gcell!');
            end
            integration_rule = get(parent,'integration_rule');
            if ~isempty(integration_rule)
                f2gmap=fenode_to_gcell_map (struct ('gcells',gcells));
                gmap=get (f2gmap,'gcell_map');
                mater =get(parent,'mater');
                self.matstates = cell(length(gmap),1);
                if (~isempty(mater))
                    for i=1:size(self.matstates,1)
                        self.matstates{i,1} = newmatstate(mater);
                    end
                end
            else
                self.matstates = [];
            end
            dim= get(gcells,'dim');%%%%%
            switch dim%%%%
                case 1
                    self.hBlmat=@feblock_defor_nonlinear_nice_Blmat1;%%%%%
                case 2
                    self.hadjugate=@feblock_defor_nonlinear_nice_adjugate2;
                    if get(gcells(1),'axisymm')
                        self.hBlmat=@feblock_defor_nonlinear_nice_Blmat2axisymm;%%%%%
                    else
                        self.hBlmat=@feblock_defor_nonlinear_nice_Blmat2;%%%%
                    end
                case 3
                    self.hadjugate=@feblock_defor_nonlinear_nice_adjugate3;
                    self.hBlmat=@feblock_defor_nonlinear_nice_Blmat3;%%%%%
                    self.hGmat=@feblock_defor_nonlinear_nice_gradient_mat3;%%%%%
                otherwise
                    error (' Not implemented');
            end
            % Precompute basis functions for quadrature points
            self.N = [];
            self.Nder= [];
            integration_rule = get(parent, 'integration_rule');
            if (~isempty( integration_rule ))
                pc = get(integration_rule, 'param_coords');
                w  = get(integration_rule, 'weights');
                npts_per_gcell = get(integration_rule, 'npts');
                for j=1: npts_per_gcell
                    self.N{j} = bfun (gcells(1), pc(j,:));
                    self.Nder{j} = bfundpar (gcells(1), pc(j,:));
                end
            end
            retobj = class(self,class_name,parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end
