% Compute the restoring force vectors
% by which the individual gcells act on the nodes.
% Return an array of elevec objects so they may be assembled.
%    Call as
% evs = restoring_force(feb, geom, u1, u), or
% [feb,evs] = restoring_force(feb, geom, u1, u)
%     geom     - geometry field
%     u1       - displacement field at the end of time step t_n+1
%     u        - displacement field at the end of time step t_n
%
% Note: This method *UPDATES* the state of the feblock object.  In
%       particular, the material state gets updated.  If this gets
%       called for converged u1, the feblock must be assigned to itself
%       on return; otherwise the first return value must be ignored!
%
function [self,evs] = restoring_force(self, geom, u1, u)
gcells = get(self.feblock,'gcells');
    ngcells =count (gcells);
    nfens = get(gcells,'nfens');
    dim = get(geom,'dim');
    mat = get(self, 'mater'); % Material
    matstates = get(self, 'matstates');
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    tangents =eye(dim);
    %     Evaluate the nodal basis function gradients
    bfun_gradients = nodal_bfun_gradients (self, geom);
    %     Preallocate
    Fes= cell(length(bfun_gradients),1);
    eqnums = cell(length(bfun_gradients),1);
    for i=1:length(bfun_gradients)
        eqnums{i} =[];
        Fes{i} = [];
    end
		Xs =gather(geom, 1:length(bfun_gradients), 'values', 'noreshape'); % coordinates of all nodes
		U1s = gather(u1, 1:length(bfun_gradients), 'values', 'noreshape'); % displacement in step n+1
		eqnumss =gather(u, 1:length(bfun_gradients), 'eqnums','noreshape');% equation numbers for all nodes
    % Now loop over all finite element nodes
    iem =1;
    for nix=1:length(bfun_gradients)
        if bfun_gradients{nix}.Vpatch>0 % This node has a element patch in this block
            np=length(bfun_gradients{nix}.patchconn);
            c = Xs(nix, :); % coordinates of central node
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,tangents);% tangent vectors?
            end
            Xnod=Xs(bfun_gradients{nix}.patchconn,:);
            U1nod=U1s(bfun_gradients{nix}.patchconn,:);
            x1nod=Xnod+U1nod;
            F1nodal= x1nod'*bfun_gradients{nix}.Nspd;
            Bnodal= self.hBlmat (self,bfun_gradients{nix}.Nspd/F1nodal,c,Rm);
            D = tangent_moduli (mat, struct ('ms',matstates{nix,1},...
                'F',F1nodal,'stiff_type','Eulerian'));
            [sigma,matstates{nix,1}] = update(mat, matstates{nix,1},...
                struct ('output','Cauchy','F',F1nodal));
            Fe = -Bnodal'* (sigma * (bfun_gradients{nix}.Vpatch *det(F1nodal))); % note the sign
            Fes{iem} =  Fe;
            eqnums{iem} =  reshape(eqnumss(bfun_gradients{nix}.patchconn,:)',(np)*size(eqnumss,2),1);
            iem=iem+1;
        end
    end
    self.matstates = matstates;
    evs  = elevecset(struct('vec',{Fes(1:iem-1)}, 'eqnums',{eqnums(1:iem-1)}));
    return;
end
