% Compute the stiffness matrices of the individual beam gcells.
% Return an array of them so they may be assembled.
%    
% function ems = stiffness (self, geom0, u1, Rfield1, dchi)
% 
%     geom0=initial geometry field; locations of nodes
%     u1=current geometry increment field; displacements of nodes
%     Rfield1=field of nodal frames, current deformation
%     dchi=field of translation and rotation increments
%             Equation numbers are taken from the field dchi.   
% 
% The stiffness matrices are computed in the local coordinate system and
% then transformed into the global coordinate system. 
% Equation numbers are taken from the field dchi. 
%
function ems = stiffness (self, geom0, u1, dchi)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells,'nfens');
    dim = get(geom0,'dim');
    conns = get(gcells, 'conn'); % connectivity
    % Pre-allocate outputs
    Ke = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        e1=gather(dchi,conns(i,1),'eqnums');
        e2=gather(dchi,conns(i,2),'eqnums');
        eqnums{i} =[e1(1:3);e2(1:3)];
        Ke{i} =[];
    end
    Xs =gather(geom0,(1:get(geom0,'nfens')),'values','noreshape');
    U1s =gather(u1,(1:get(u1,'nfens')),'values','noreshape');
    X1s=Xs+U1s;
    % Now loop over all gcells in the block
    for i=1:ngcells
        conn = conns(i,:); % connectivity
        x0 = Xs(conn,:); % initial coordinates of nodes
        x1 = X1s(conn,1:3); % current coordinates of nodes
        [L0,L1,Ft]=local_frames(x0,x1);
        k=self.Stiffness_function(L0,L1);
        K11=k*(Ft(:,1)*Ft(:,1)');
        Ke{i} = [K11,-K11;-K11,K11];% 
    end
    ems  = elematset(struct('mat',{Ke}, 'eqnums',{eqnums}));
end