% Inspect the integration point quantities.
%
% function idat = inspect_integration_points(self, ...
%         geom, u, dT, gcell_list, context,...
%         inspector, idat)
%
% Input arguments
%    geom - reference geometry field
%    u - displacement field
%    dT - temperature difference field
%    gcell_list - indexes of the geometric cells that are to be inspected:
%          The gcells to be included are: gcells(gcell_list).
%    context    - structure: see the update() method.
%    inspector - function handle or in-line function with the signature
%             idat =inspector(idat, out, xyz, pc),
%        where
%     idat - a structure or an array that the inspector may
%                use to maintain some state, for instance minimum or
%                maximum of stress, out is the output  of the update()
%                method, and xyz is the location of the integration point
%                in the *reference* configuration. The argument pc are the
%                parametric coordinates of the quadrature point.
% Output arguments
%     idat - see the description of the input argument
%
function idat = inspect_integration_points(self, ...
        geom, u, dT, gcell_list, context,...
        inspector, idat)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    conns = get(gcells, 'conn'); % connectivity
   labels = get(gcells, 'label'); % connectivity
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    % Material
    mat = get(self.feblock, 'mater');
    matstates = get(self, 'matstates');
    context.strain= [];
    xs =gather(geom,(1:get(geom,'nfens')),'values','noreshape');
    if isempty(dT)
        dTs=zeros(get(geom,'nfens'),1);
    else
        dTs=gather(dT, (1:get(geom,'nfens')), 'values', 'noreshape');
    end
    % Now loop over selected gcells 
    for m=1:length(gcell_list)
        i=gcell_list(m);
        conn = conns(i,:); % connectivity
        X = xs(conn,:);
        U = gather(u,conn,'values');
        dT =dTs(conn,:);
        % Loop over all integration points
        for j=1:npts_per_gcell
            conn =conns(i,:);
            x=xs(conn,:);
            c =Ns{j}'*x;% physical location
            J = x' * Nders{j};%J = Jacobian_matrix(gcells,Nders{j},x);%J = x' * Nder;
            if (eval_Rm)% do I need to evaluate the local material orientation?
                if (~isempty(labels )),  Rm =Rmh(c,J,labels(i));
                else,                    Rm =Rmh(c,J,[]);                end
            end
            Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x);
            if (isempty(Rm))
                Ndersp = Nders{j}/J;% derivatives wrt local coor
            else
                Ndersp = Nders{j}/(Rm'*J);% derivatives wrt local coor
            end
            B = self.hBlmat(self,Ns{j},Ndersp,c,Rm);% strain-displacement
            context.strain = B*U;
            context.dT = Ns{j}'*dT;
            context.xyz =c;
            [out,ignore] = update(mat, matstates{i,j}, context);
            if ~isempty (inspector)
                idat =feval(inspector,idat,out,c,pc(j,:));
            end
        end
    end
    return;
end
