% Compute the thermal-strain load vectors of the individual biaxial gcells.
%
% function evs = thermal_strain_loads(self, geom, u, dT)
%
% Return an array of the element vectors so they may be assembled.
%    Call as
% ems = thermal_strain_loads(feb, geom, u, T)
%     geom=geometry field
%     u=displacement field
%     dT=temperature difference field (temperature at a point minus the
%     reference temperature at which the solid experiences no strains)
%
function evs = thermal_strain_loads(self, geom, u, dT)
    gcells = get(self,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    conns = get(gcells, 'conn'); % connectivity
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    % Material
    mat = get(self, 'mater');
    matstates = get(self, 'matstates');
    %     Preallocate
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Fe{i} =zeros(get(geom,'dim')*nfens,1);
    end
    xs =gather(geom,(1:get(geom,'nfens')),'values','noreshape');
    % Now loop over all gcells in the block
    for i=1:ngcells
        conn =conns(i,:);
        x=xs(conn,:);
        U = gather(u, conn, 'values'); % displacements
        dTs = gather(dT, conn, 'values'); % displacements
        % Loop over all integration points
        for j=1:npts_per_gcell
            c =Ns{j}'*x;
            J = x' * Nders{j};%J = Jacobian_matrix(gcells,Nders{j},x);%J = x' * Nder;
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,J);
            end
            if (isempty(Rm))
                Ndersp = Nders{j}/J;% derivatives wrt local coor
            else
                Ndersp = Nders{j}/(Rm'*J);% derivatives wrt local coor
            end
            Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x);
            B = self.hBlmat(self,Ns{j},Ndersp,c,Rm);% strain-displacement
            context.D = tangent_moduli(mat,struct('xyz',c));
            context.ms =matstates{i,j};
            context.dT =Ns{j}'*dTs; context.xyz =c;
            iSigma = thermal_stress(mat,context);
            Fe{i} = Fe{i} - B'*(iSigma * (Jac * w(j)));
        end
    end
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
end
