% Compute the acoustic pressure-structure coupling matrix. It transforms 
% the pressure distributed along the surface to forces acting on the nodes
% of the finite element model. Its transpose transforms displacements (or velocities, or
% accelerations) into the normal component of the displacement (or
% velocity, or acceleration) along the surface.
%
% function ems = acoustic_coupling_panels_antisymm (self, geom, u, ell)
%
% Return an array of the element matrices so they may be assembled.
%     
% Arguments
%     geom=geometry field
%     u = displacement field 
%     ell= ell-th harmonic, ell=0,1,2,
%
% Notes:
% -- n=outer normal (pointing into the acoustic medium).
% -- The pressures along the surface are assumed constant (uniform) along 
% each finite element - gcell (panel). The panel pressures are assumed to
% be given the same numbers as the serial numbers of the gcells in the block.
%
function ems = acoustic_coupling_panels_antisymm (self, geom, u, ell)
    if (ell==0)
        csmultiplier=0; % cosine products give 2*pi, sine products give zero
    else
        csmultiplier=1/2; % cosine, sine products give pi for ell >0
    end
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells,'nfens');
    dim = get(u,'dim');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Precompute basis f. values + basis f. gradients wrt parametric coor
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Prepare some data: connectivity and the element matrices
    conns = get(gcells, 'conn'); % connectivity
    xs =get(geom,'values');% Retrieve Geometry information
    Ge = cell(size(conns,1),1);
    eqnums_row = cell(size(conns,1),1);
    eqnums_col = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums_row{i} =gather(u,conns(i,:),'eqnums');
        eqnums_col{i} =i;
        Ge{i} =zeros(dim*nfens,1);
    end
    xs =get(geom,'values');
    for i=1:size(conns,1)
        conn =conns(i,:);
        x=xs(conn,:);
        for j=1:npts_per_gcell
            Y=transpose(Ns{j})*x; % Point at which pressure force acts
            J = Jacobian_matrix(gcells,Nders{j},x);
            Jac = Jacobian_surface(gcells,conn, Ns{j}, J, x);
            n= normal(self,Y,J);
            Ge{i} = Ge{i} +  reshape(n*transpose(Ns{j}),dim*nfens,1)*((csmultiplier*Jac*w(j)));
        end
    end
    ems  = elematset(struct('mat',{Ge}, 'eqnums_row',{eqnums_row},'eqnums_col',{eqnums_col}));
end
