% Constructor of block for small displacement, small strain deformation 
% of axially symmetric solids undergoing general three-dimensional motion.
%
% function self = feblock_defor_ss_3daxi (varargin)
%
% Constructor of the finite element block class to be used with small
% displacement, small strain    deformation mechanics for continuum,
% isoparametric elements.
%
% Reference:
%  ANALYSIS OF AXISYMMETRIC STRUCTURES UNDER ARBITRARY LOADING USING THE 
%  HYBRID-STRESS  MODEL 
%  ROBERT  L.  SPILKER AND  DIANE  M.  DAUGIRDAS
%  INTERNATIONAL JOURNAL FOR NUMERICAL METHODS IN ENGINEERING,  VOL. 17, 801-828 (1981)  
%
% See discussion of constructors in <a href="matlab:helpwin 'FAESOR/classes/Contents'">classes/Contents</a>. 
% Options: those recognized by feblock, and
% N = Outer normal, either as an array of numbers,or as a function handle.
%     Signature as in this example (tangent is in matrix with tangent
%     vectors as columns).
%     function n= ABC_normal(x,tangent)
%         n= cross(tangent(:,1),tangent(:,2));
%         n=n/norm(n);
%     end
%
function self = feblock_defor_ss_3daxi (varargin)
    class_name='feblock_defor_ss_3daxi';
    if (nargin == 0)
        parent = feblock;
        self.matstates= [];
        self.hBlmat= [];
        self.hBlmat_0_symm= [];
        self.hBlmat_0_antisymm= [];
        self.N= [];
        self = class(self,class_name,parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            self = arg;
            return;
        else
            Parameters=varargin{1};
            parent = feblock(varargin{:});
            self.matstates = [];
            self.hBlmat= [];
            self.hBlmat_0_symm= [];
            self.hBlmat_0_antisymm= [];
            self.N= [];% 
            if isfield(Parameters,'N')
                self.N = Parameters.N;
            end
            self = class(self,class_name,parent);
            integration_rule = get(self.feblock, 'integration_rule');
            npts_per_gcell = get(integration_rule, 'npts_per_gcell');
            gcells = get(self.feblock,'gcells');
            ngcells = count(gcells);
            self.matstates = cell(get(gcells(1),'nfens')*ngcells,npts_per_gcell);
            mat = get(self.feblock, 'mater');
            for i=1:ngcells
                for j=1:npts_per_gcell;
                    self.matstates{i,j} = newmatstate(mat);
                end
            end
            gcells = get(self,'gcells');
            dim= get(gcells(1),'dim');
            switch dim
                case 1
                    if get(gcells,'axisymm')
                        self.hBlmat=[];
                        self.hBlmat_0_antisymm=[];
                        self.hBlmat_0_symm=[];
                    else
						error ('Wrong geometry cells: not axisymmetric');
                    end
                case 2
                    if get(gcells,'axisymm')
                        self.hBlmat=@feblock_defor_ss_3daxi_Blmat;
                        self.hBlmat_0_antisymm=@feblock_defor_ss_3daxi_Blmat_0_antisymm;
                        self.hBlmat_0_symm=@feblock_defor_ss_3daxi_Blmat_0_symm;
                    else
						error ('Wrong geometry cells: not axisymmetric');
                    end
                case 3
                    error (' Wrong geometry cells');
                otherwise
                    error (' Wrong geometry cells');
            end
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end

