% Compute the strain-displacement matrix for a two-manifold element for
% axially symmetric models undergoing general three-dimensional motion in cylindrical coordinates.
%
% B = feblock_defor_ss_3daxi_Blmat(self,N,Ndersp,c,Rm)
%
% Compute the linear, displacement independent, strain-displacement matrix
% for a two-manifold element for axially symmetric models.
% 
% Reference:
%  ANALYSIS OF AXISYMMETRIC STRUCTURES UNDER ARBITRARY LOADING USING THE 
%  HYBRID-STRESS  MODEL 
%  ROBERT  L.  SPILKER AND  DIANE  M.  DAUGIRDAS
%  INTERNATIONAL JOURNAL FOR NUMERICAL METHODS IN ENGINEERING,  VOL. 17, 801-828 (1981)  
% 
% Note:  the terms of the strains  have been reordered to match the 3-D
% strains  in the FAESOR toolkit.
% Therefore, the symmetric displacement matrix 
% C_Bar  =diag(  [cos(ell*theta),cos(ell*theta),-sin(ell*theta)] )
% and the  anti-symmetric matrix is
% C_Barbar  =diag(  [sin(ell*theta),sin(ell*theta),cos(ell*theta)] )
%
%      Ndersp=matrix of basis function gradients
%      ell= ell-th harmonic, ell=0,1,2,
%      c=array of spatial coordinates of the evaluation point
%      Rm=orthogonal matrix with the unit basis vectors of the local
%         coordinate system as columns; supply Rm as empty when there
%         is no need for global-to-local transformation (for instance for
%         isotropic materials).
%         size(Rm)= [ndim,ndim], where ndim = Number of spatial dimensions
%         of the embedding space.
% Output: B = strain-displacement matrix, size (B) = [nstrain,nfens*dim],
%      where nstrain= number of strains, dim = Number of spatial dimensions of
%      the embedding space, and nfens = number of finite element nodes on
%      the element.
%
%  NOTE: the displacements components are assumed to be in the order u_r, u_z, u_theta, 
%  (that is radial, axial, and circumferential).  The strains are in the order
%  epsilon_r, epsilon_theta, epsilon_z, gamma_rz, gamma_ztheta, gamma_rtheta.
%
function B = feblock_defor_ss_3daxi_Blmat_0_symm(self, N, Ndersp, location, Rm)
    ell=0; % this is the assumption  here
    nfn= size(Ndersp,1);
    r=location(1); if r==0,r=eps; end
    dim =3;
    B = zeros(6,nfn*dim);
    if (isempty(Rm))% there is no global-to-local transformation
        for i= 1:nfn % u_r             u_z                    u_theta 
            B(:,dim*(i-1)+1:dim*i)=...
               [Ndersp(i,1)           0 					   0; ... 
                0     			      Ndersp(i,2)              0; ... 
                N(i)/r                0                        -ell*N(i)/r; ...
                Ndersp(i,2)           Ndersp(i,1)              0; ... 
                0                     0                        0; ... 
                0                     0                        0];% note that the 
                % last two rows  are all zeros: only the cosine--multiplied terms survive for ell==0
        end
    else% global-to-local transformation is requested
        for i= 1:nfn
            B(:,dim*(i-1)+1:dim*i)=...
               [Ndersp(i,1)           0 					   0; ... 
                0     			      Ndersp(i,2)              0; ... 
                N(i)/r                0                        -ell*N(i)/r; ...
                Ndersp(i,2)           Ndersp(i,1)              0; ... 
                0                     0                        0; ... 
                0                     0                        0]*Rm';% note that the 
                % last two rows  are all zeros: only the cosine--multiplied terms survive for ell==0
        end
    end
end
