% Compute the stiffness matrices of the individual gcells.
%
% function ems = stiffness (self, geom, u, ell)
%
% Return an array of the element matrices so they may be assembled.
%     geom=geometry field
%     u=displacement field
%     ell= ell-th harmonic, ell=1,2, 
% The symmetric and anti-symmetric give identical stiffness.
%
function ems = stiffness (self, geom, u, ell)
    if (ell==0)
    	error (['Cannot handle ell==0 here']);
    end
    csmultiplier=1/2; % cosine, sine products give pi for ell >0
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells,'nfens');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    % Material
    mat = get(self.feblock, 'mater');
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Ke = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Ke{i} =zeros(get(u,'dim')*nfens);
    end
    xs =gather(geom,(1:get(geom,'nfens')),'values','noreshape');
    for i=1:size(conns,1)
        conn =conns(i,:);
        x=xs(conn,:);
        for j=1:npts_per_gcell
            c =Ns{j}'*x;% physical location of the quadrature point
            J = x' * Nders{j};% We compute the Jacobian matrix
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,J);
            end
            if (isempty(Rm))
                Ndersp = Nders{j}/J;% derivatives wrt global coor
            else
                Ndersp = Nders{j}/(Rm'*J);% derivatives wrt local coor
            end
            Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x);
            if (Jac<=0),error('Non-positive Jacobian');end
            B = self.hBlmat(self,Ns{j},Ndersp,ell,c,Rm);% strain-displacement
            D = tangent_moduli(mat,struct('xyz',c));
            Ke{i} = Ke{i} + (B'*(D*(csmultiplier*Jac*w(j)))*B);
        end
    end
    ems  = elematset(struct('mat',{Ke}, 'eqnums',{eqnums}));
end
