% Compute the element load vectors.
%
% function evs = distrib_loads(self, geom, u, fi, m)
%
% Compute the element load vectors corresponding to applied distributed
% load. Here it means force per unit volume where volume could be
% length^3, length^2, length^1, or length^0, depending on the manifold
% dimension of the geometric cells.
% Return an array of the load vectors so they may be assembled.
%    Call as
% evs = distrib_loads(feb, geom, u, fi, m)
%     geom=geometry field
%     u=displacement field
%     fi=force intensity object
%     m= manifold dimension
%
function evs = distrib_loads(self, geom, u, fi, m)
     gcells = get(self.feblock,'gcells');
    nfens = get(gcells(1),'nfens');
    dim = get(u,'dim');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Fe{i} =zeros(get(geom,'dim')*nfens,1);
    end
    xs =get(geom,'values');
    for j=1:npts_per_gcell
        N = bfun(gcells,pc(j,:));
        Nder = bfundpar(gcells,pc(j,:));
        for i=1:size(conns,1)
            conn =conns(i,:);
            x=xs(conn,:);
            J = Jacobian_matrix(gcells,Nder,x);
            Jac = Jacobian_mdim(gcells,conn, N, J, x, m);
            f=get_magn(fi,N'*x,J);
            Fe{i} = Fe{i} + reshape(f*N',dim*nfens,1) * (Jac * w(j));
        end
    end
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
end