% Inspect the integration point quantities.
%
% function idat = inspect_integration_points(self, ...
%         geom, u, dT, gcell_list, context,...
%         inspector, idat)
%
% Input arguments
%    x - reference geometry field
%    u - displacement field
%    dT - temperature difference field
%    gcell_list - indexes of the geometric cells that are to be inspected:
%          The gcells to be included are: gcells(gcell_list).
%    context    - structure: see the update() method.
%    inspector - function handle or in-line function with the signature
%             idat =inspector(idat, out, xyz, pc),
%        where
%     idat - a structure or an array that the inspector may
%                use to maintain some state, for instance minimum or
%                maximum of stress, out is the output  of the update()
%                method, and xyz is the location of the integration point
%                in the *reference* configuration. The argument pc are the
%                parametric coordinates of the quadrature point.
% Output arguments
%     idat - see the description of the input argument
%
function idat = inspect_integration_points(self, ...
        x, u, dT, list, context,...
        inspector, idat)
    gcells = get(self.feblock,'gcells');
    dim = get(geom,'dim');
    % Integration rule
    integration_rule = get(self, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts=get(integration_rule, 'npts');
    for j=1:size(pc,1)
        N{j} = bfun(gcells, pc(j,:));
        Nder{j} = bfundpar(gcells, pc(j,:));
    end
    % Material
    mat = get(self.feblock, 'mater');
    D = tangent_moduli(mat,struct([]));
    [Princd,W]=eig(D);
    [ignore,ix]=sort(diag(W));
    Wmean=(W(ix(1),ix(1))+W(ix(2),ix(2)))/2;
    if Wmean<=0
        Wmean=max(diag(W));
    end
    % Make the inverse map from finite element nodes to geometric cells
    gmap=get (self.f2gmap,'gcell_map');
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    xs =gather(geom, 1:length(gmap), 'values', 'noreshape'); % coordinates of all nodes
    % Now loop over all finite element nodes
    lnmap =0*(1:length(gmap));
    fld =0*slice(u,1,'fld');
    for nix=1:length(gmap)
        gl=gmap{nix};
        thisnn=nix;
        if ~isempty(gl) % This node has a element patch in this block
            % establish local numbering of all nodes of the patch @node nix
            kconns = conns(gl,:); % connectivity
            patchconn = patch_conn (self,kconns,thisnn);
            np =length(patchconn);
            lnmap(patchconn) =1:np;% now store the local numbers
            c = xs(thisnn, :); % coordinates of central node
            D = tangent_moduli (mat, struct ('xyz',c));% material tangent
            Rm = material_directions(self,gcells,[],c);
            Nspdavg=zeros(np,dim);% preallocate strain-displacement matrix
            Vpatch =0;
            for k=1:length(gl)
                kconn =kconns(k,:);
                pci=find(kconn == thisnn);% at which node in the element are we?
                if (pci<=npts)
                    px = xs(kconn, :); % coordinates of nodes
                    lx=px-ones(length(kconn),1)*c;% centered
                    xl=lx*Rm;% local coordinates
                    J = Jacobian_matrix(gcells,Nder{pci},xl);
                    Jac = Jacobian_volume(gcells,kconn, N{pci}, J, xl);
                    Nspd = Nder{pci}*self.hadjugate(J);
                    Vpatch=Vpatch+Jac*w(pci);
                    Nspdavg(lnmap(kconn),:)=Nspdavg(lnmap(kconn),:)+w(pci)*Nspd;
                end
            end
            Nspdavg=Nspdavg/ Vpatch;
            Bnodal= self.hBlmat (self,Nspdavg,c,Rm);
            U = gather(u, conn, 'values'); % displacement
            context.strain = Bnodal*U ;
            context.dT = dTs;
            context.xyz =c;
            [out,ignore] = update(mat, [], context);
            if ~isempty (inspector)
                idat =feval(inspector,idat,out,c,pc(j,:));
            end
        end
        lnmap(patchconn) = 0;
    end
end

%
%
% function idat = inspect_integration_points(self, ...
%         x, u, dT, list, context,...
%         inspector, idat)
%     gcells = get(self.feblock,'gcells');
%     dim = get(x,'dim');
%     % Integration rule
%     integration_rule = get(self, 'integration_rule');
%     pc = get(integration_rule, 'param_coords');
%     w  = get(integration_rule, 'weights');
%     npts=get(integration_rule, 'npts');
%     nconn =length(get(gcells(1), 'conn'));
%     % Material
%     mat = get(self.feblock, 'mater');
%     D = tangent_moduli(mat,struct([]));
%     [N,W]=eig(D);
%     [ignore,ix]=sort(diag(W));
%     Wmean=(W(ix(1),ix(1))+W(ix(2),ix(2)))/2;
%     if Wmean<=0
%         Wmean=max(diag(W));
%     end
%     context.gradU= [];
%     % Make the inverse map from finite element nodes to geometric cells
%     f2gmap=fenode_to_gcell_map (struct ('gcells',gcells));
%     gmap=get (f2gmap,'gcell_map');
%     % Now loop over all gcells in the block
%     fld =slice(x,1,'fld');
%     nvals = gather (fld, (1: get(fld,'nfens')),'values');
%     for nix=1:length(gmap)
%         gl=gmap{nix};
%         thisnn=nix;
%         if ~isempty(gl) % This node has a element patch in this block
%             % establish local numbering of all nodes of the patch @node nix
%             lnmap =0*(1:length(gmap));
%             for k=1:length(gl)
%                 kconn = get(gcells(gl(k)), 'conn'); % connectivity
%                 for j= 1:length(kconn)
%                     if (lnmap(kconn(j)) == 0) & (kconn(j) ~= thisnn)
%                         lnmap(kconn(j))=kconn(j);
%                     end
%                 end
%             end
%             patchconn=lnmap(lnmap>0);
%             % now store the local numbers
%             for k=1:length(patchconn)
%                 lnmap(patchconn(k)) =k;
%             end
%             lnmap(thisnn) =length(patchconn)+1;
%             c = gather(x, thisnn, 'values', 'noreshape'); % coordinates of central node
%             if isempty(dT)
%                 dTs=0;
%             else
%                 dTs = gather(dT, thisnn, 'values', 'noreshape'); % displacement
%             end
%             % material tangent
%             D = tangent_moduli (mat, struct ('xyz',c));
%             Rm = material_directions(self,gcells(gl(1)),[],c);
%             % preallocate the strain-displacement matrix
%             Nspatialderavg=zeros((length(patchconn)+1),dim);
%             Vpatch =0;
%             for k=1:length(gl)
%                 kconn = get(gcells(gl(k)), 'conn'); % connectivity
%                 lx = gather(x, kconn, 'values', 'noreshape'); % coordinates of nodes
%                 lx=lx-ones(length(kconn),1)*c;
%                 % get the parametric coordinates
%                 pci=find(kconn == thisnn);% at which node in the element are we?
%                 thisnn_pc=pc(pci,:); % parametric coordinates of that node
%                 Nder = bfundpar(gcells(gl(k)), thisnn_pc);
%                 xl=lx*Rm;% local coordinates
%                 Nspatialder = bfundsp (gcells(gl(k)), Nder, xl);
%                 detJ = Jacobian_volume(gcells(gl(k)), thisnn_pc,xl);
%                 Ve = detJ*w(pci);
%                 Vpatch=Vpatch+Ve;
%                 Nspatialderavg(lnmap(kconn),:)=Nspatialderavg(lnmap(kconn),:)+Ve*Nspatialder;
%             end
%             Nspatialderavg=Nspatialderavg/Vpatch;
%             Bnodal= self.hBlmat (self,Nspatialderavg,c,Rm);
%             conn= [patchconn thisnn];
%             U = gather(u, conn, 'values'); % displacement
%             context.strain = Bnodal*U ;
%             context.dT = dTs;
%             context.xyz =c;
%             [out,ignore] = update(mat, [], context);
%             if ~isempty (inspector)
%                 idat =feval(inspector,idat,out,c,pc(j,:));
%             end
%         end
%     end
% end
%
