% Compute the conductivity matrices of the individual gcells.
%
% function ems = conductivity(self, geom, temp)
%
% Return an array of the element matrices so they may be assembled.
%    Call as     ems = conductivity(feb, geom, temp)
%    where   geom=geometry field
%            temp=temperature field
function ems = conductivity(self, geom, temp)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells,'nfens');
    dim = get(geom,'dim');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Precompute basis f. values + basis f. gradients wrt parametric coor
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material
    mat = get(self.feblock, 'mater');
    kappa = get(get(mat,'property'),'conductivity');
    % Prepare some data: connectivity and the element matrices
    conns = get(gcells, 'conn'); % connectivity
    Ke = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(temp,conns(i,:),'eqnums');
        Ke{i} =zeros(nfens);
    end
    xs =get(geom,'values');% retrieve the geometry information
    for i=1:size(conns,1)
        conn =conns(i,:);
        x=xs(conn,:);
        for j=1:npts_per_gcell
            J = Jacobian_matrix(gcells,Nders{j},x);
            Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x);
            Ndersp = Nders{j}/J;
            Rm = material_directions(self,gcells,pc(j,:),x);
            Ke{i} = Ke{i} + Ndersp*Rm*(kappa*Jac*w(j))*Rm'*Ndersp' ;
        end% Loop over quadrature points
    end% Loop over elements
    ems  = elematset(struct('mat',{Ke}, 'eqnums',{eqnums}));
end

% Here is an example of the order of the loops reversed, first over the 
% quadrature points, then over the geometric cells:
% for j=1:npts_per_gcell
%         N = bfun(gcells,pc(j,:));
%         Nder = bfundpar(gcells,pc(j,:));
%         for i=1:size(conns,1)
%             conn =conns(i,:);
%             x=xs(conn,:);
%             J = Jacobian_matrix(gcells,Nder,x);
%             Jac = Jacobian_volume(gcells,conn, N, J, x);
%             Ndersp = Nder/J;
%             Rm = material_directions(self,gcells,pc(j,:),x);
%             Ke{i} = Ke{i} + Ndersp*Rm*(kappa*Jac*w(j))*Rm'*Ndersp' ;
%         end
%     end