% Update material state.  Return the updated material state, and the
% requested quantity (default is the stress).
%   Call as:
%     [out,newms] = update(m, ms, context)
%  where
%     m=material
%     ms = material state
%     context=structure
%        with mandatory fields
%           strain = strain vector
%        and optional fields
%           output=type of quantity to output, and interpreted by the
%           particular material; [] is returned when the material does not
%           recognize the requested quantity to indicate uninitialized
%           value.  It can be tested with isempty ().
%              output ='Cauchy' - Cauchy stress; this is the default
%                      when output type is not specified.
%              output ='2ndPK' - 2nd Piola-Kirchhoff stress;
%           It is assumed that stress is output in 3-component vector form
%           for plane stress, and four-component vector form for plane strain.
%   The output arguments are
%     out=requested quantity
%     newms=new material state; don't forget that if the update is final
%           the material state newms must be assigned and stored.  Otherwise
%           the material update is lost!
%
function [out, newms] = update (self, ms, context)
    E = get(self.property,'E', context);;
    nu = get(self.property,'nu', context);;
    sigma_y = get(self.property,'sigma_y', context);;
    lambda = E * nu / (1 + nu) / (1 - 2*(nu));          % Lame constant #1
    mu     = E / (2 * (1 + nu));                        % shear modulus
    kappa  = E / 3 / (1 - 2*nu);                        % bulk modulus
    mI = diag([1 1 1 0.5 0.5 0.5]);
    m1     = [1 1 1 0 0 0]';
    eps1 = context.strain;                        % total strain
    e1   = eps1 - 1/3*(eps1'*m1)*m1;             % deviatoric total strain
    s1_trial= stress_6v_to_3x3t (self,2 * mu * mI * (e1 - ms.eps_p));
    norm_s1_trial = sqrt(trace(s1_trial*s1_trial));
    f1_trial = norm_s1_trial - sqrt(2/3)*sigma_y;  % trial yield f. value
    sigma1 = kappa * (eps1'*m1) * eye(3,3) + s1_trial;       % trial stress
    if (f1_trial > 0)                                   % plastic correction
        n1 = s1_trial/norm_s1_trial;                    % unit normal to f1_trial
        dgamma1 = f1_trial / (2 * mu);                  %
        sigma1 = sigma1 - 2 * mu * dgamma1 * n1;        % correct stress
        ms.eps_p = ms.eps_p + dgamma1 * strain_3x3t_to_6v (self,n1);    % update material state
        ms.dgamma = dgamma1;
        ms.equiv_pl_def = ms.equiv_pl_def + sqrt(2/3) * ms.dgamma;
    end
    sigma1 =stress_3x3t_to_6v (self,sigma1);
    ms.strain_energy = ms.strain_energy + 1/2*sigma1'*(eps1 - ms.eps_p);
    stress = sigma1;
    tSigma = thermal_stress(self,context);
    stress = stress + tSigma;
    if isfield(context,'output')
        switch context.output
            case 'Cauchy'
                out = stress;
            case 'pressure'
                out = -sum(stress(1:3))/3;
            case 'vol_strain'
                out = sum(ve1(1:3)); 
            case 'equiv_pl_def'
                out =ms.equiv_pl_def;
            otherwise
                out = [];
        end
    else
        out = stress;
    end
    newms = ms;
    return;
end
% % update: Update material state.  Return the stress.
% %   Call as:
% %     [S,newms] = update(m,ms,F1,F,stress_type)
% %   where
% %     S=stress in 6-component vector form
% %     newms=new material state; don't forget that if the update is final
% %           the material state newms must be assigned and stored.  Otherwise
% %           the material update is lost!
% %     m=material
% %     ms=material state
% %     F1=deformation gradient at time t+dt
% %     F=deformation gradient at time t
% %     stress_type=Cauchy ('Cauchy'), 2nd Piola-Kirchhoff ('2ndPK')
% %
% function [stress, newms] = update (self, ms, F1, F, stress_type)
%     E = get(self.property,'E', context);;
%     nu = get(self.property,'nu', context);;
%     lambda = E * nu / (1 + nu) / (1 - 2*(nu));          % Lame constant #1
%     mu     = E / (2 * (1 + nu));                        % shear modulus
%     kappa  = E / 3 / (1 - 2*nu);                        % bulk modulus
%     mI = diag([1 1 1 0.5 0.5 0.5]);
%     m1     = [1 1 1 0 0 0]';
%     gradU = F1 - eye(3,3);                              % displacement gradient
%     eps1 = 1/2*(gradU + gradU');                        % total strain
%     e1   = eps1 - 1/3*trace(eps1)*eye(3,3);             % deviatoric total strain
%     ve1  = strain_tensor_to_6_vector (self.mater,e1);
%     s1_trial = 2 * mu * mI * (ve1 - ms.eps_p);          % trial deviatoric stress
%     norm_s1_trial = sqrt(s1_trial(1:3)'*s1_trial(1:3)+2*s1_trial(4:6)'*s1_trial(4:6));
%     f1_trial = norm_s1_trial - sqrt(2/3)*self.sigma_y;  % trial yield f. value
%     sigma1 = kappa * trace(eps1) * m1 + s1_trial;       % trial stress
%     if (f1_trial > 0)                                   % plastic correction
%         n1 = s1_trial/norm_s1_trial;                      % unit normal to f1_trial
%         dgamma1 = f1_trial / (2 * mu);                    %
%         sigma1 = sigma1 - 2 * mu * dgamma1 * mI * n1;     % correct stress
%         ms.eps_p = ms.eps_p + dgamma1 * n1;               % update material state
%         ms.dgamma = dgamma1;
%         ms.equiv_pl_def = ms.equiv_pl_def + sqrt(2/3) * ms.dgamma;
%     end
%     veps1 = strain_tensor_to_6_vector(self.mater,eps1);
%     ms.strain_energy = ms.strain_energy + 1/2*sigma1'*(veps1 - ms.eps_p);
%     stress = sigma1;
%     newms = ms;
%     return;
% 
% 
