% Constructor of a square sparse system matrix. This class provides methods for
% constructing a sparse-storage system matrix, factorization into a lower
% triangular matrix, and solution of the system of linear equations.
%
% function retobj = sparse_chol_sysmat ()
%
% Parameters: none
% 
% Notes:
% -- This class does not support multiple assemble() calls. For instance, 
% it is not possible to do
%           A= assemble(A,ems1);
%           A= assemble(A,ems2);
% Rather, the above must be accomplished by concatenating the element 
% matrix objects and calling assemble() only once
%           A= assemble(A,cat(2,ems1,ems2));
% 
% -- This class optimizes multiple solutions with different right hand sides
% as it computes the Cholesky factorization of the system matrix at the
% time of the first solve, and then just does the forward and backward
% substitution for subsequent solves. Note that the factorization is 
% stored in-place (it overwrites the original matrix).
% 
% -- Only symmetric matrices are supported.  The matrix lower left
% triangle is stored under the assumption that the matrix is indeed symmetric. 
% 
% -- The forward/backward substitution can take advantage of the
% memory-conserving solvers of the package CXSparse, provided this package
% is installed.
% 
% See discussion of constructors in <a href="matlab:helpwin 'FAESOR/classes/Contents'">classes/Contents</a>.
%
function self = sparse_chol_sysmat (varargin)
    class_name='sparse_chol_sysmat';
    parent=sysmat;
    Factorized =false;
    invalid_eqnum = get(sysmat, 'invalid_eqnum');
    A = [];% The matrix
    prm = []; % permutation vector
    neqns= 0;
    Have_CXSparse = ~ (strcmp('',which ('cs_lsolve')));
    if (Have_CXSparse)
        try
            cs_lsolve(sparse([1]),[1]);
        catch
            Have_CXSparse = false ;
        end
    end
    function assemble_method (ems)
        ne =length(ems);
        sl=0;
        for m = 1:ne
            eqnums=get(ems(m), 'eqnums');
            for j = 1:length(eqnums)
                sl =sl+length(eqnums{j})+round(1/2*length(eqnums{j})^2)+1;
            end
        end
        sppatti= zeros (sl,1);
        sppattj= zeros (sl,1);
        Ainit= zeros (sl,1);
        triplet =1;
        for m = 1:ne
            eqnums=get(ems(m), 'eqnums');
            mat    = get(ems(m), 'mat');
            if placesymm(ems(m))
                for j = 1:length(eqnums)
                    emask=eqnums{j}~=invalid_eqnum;
                    conns =eqnums{j} (emask);
                    amat =(mat{j}(emask,emask)+mat{j}(emask,emask)')/2;
                    for c= 1:length(conns)
                        for r= 1:length(conns)
                            gr =conns(r); gc=conns(c);
                            if (gr>=gc)
                                sppatti(triplet) =gr;
                                sppattj(triplet) =gc;
                                Ainit(triplet) =amat(r,c);
                                triplet =triplet +1;
                            end
                        end
                    end
                end
            else
                error ('Not implemented for unsymmetric matrices');
            end
        end
        %          Make the sparse matrix from triplets
        A=sparse(sppatti(1:triplet-1), sppattj(1:triplet-1), Ainit(1:triplet-1), neqns, neqns);
        
        return;
    end
    self.assemble_method=@assemble_method;

    function assemble_diag_method (D)
        n=size(A);
        for k = 1:n
            A(k,k)=A(k,k) +D(k);
        end
        return;
    end
    self.assemble_diag_method=@assemble_diag_method;

    % If the method is called with V empty the purpose is just factorization.
    function x=solve_method (V)
        if (~Factorized)
            [A,Status,prm] = chol(A,'lower','vector');
            disp( 'Debug***'),nnz(A), %figure;spy(A)
            Factorized = true;
        end
        if (isempty(V)) % If the method is called with V empty the purpose is just factorization.
            x = []; return; 
        end
        if (Have_CXSparse) % The functions  used below  avoid transpose of the matrix A
            x(prm) = cs_ltsolve (A, cs_lsolve (A, V(prm))); 
        else
            x(prm) = A'\(A\V(prm));% This operation allocates additional storage for A'
        end
        %
    end
    self.solve_method=@solve_method;
    
    function retobj=start_method (dim)
        neqns =dim;
        Factorized = ~true;
        A = [];
        retobj =self;
    end
    self.start_method=@start_method;

    function val = get_method(varargin)
        if (nargin == 0)
            val = {{{'dim'}, {'dimension, array '}};
                {{'mat'}, {'sparse data matrix, array [dim,dim]'}}};
            return;
        else
            prop_name=varargin{1};
            switch prop_name
                case 'mat'
                    if ( Factorized )
                        val = A;
                    else
                        val = A+A'-spdiags(spdiags(A,0),0,size(A,1),size(A,2));
                    end
                case 'dim'
                    val = neqns;
                otherwise
                    error(['Unknown property name ''' prop_name '''!']);
            end
        end
        return;
    end
    self.get_method=@get_method;

    function retval=set_method(varargin)
        if (nargin == 1)
            retval = {};
            return;
        end
        prop_name=varargin{1};
        val=varargin{2};
        switch prop_name
            otherwise
                error(['Unknown property ''' prop_name '''!'])
        end
        retval=self; % return self
    end
    self.set_method=@set_method;

    self = class(self, class_name);
end
