% Analytical solutions for free vibration of laminated composite and sandwich
% plates based on the higher-order refined theory, T. Kant and K. Swaminathan,
% Composite Structures 53  (2001) 73-85



function KS_laminate_vibration_ss

% Material: The data from the above cited article. The ratio of the Young's
% moduli is variable, and the number of layers also varies.
% E1/E2 = 3, 10, 40

% One particular case from the paper:
L_t =5;
E1_E2 = 3;% this ratio is variable
% Anti-symmetric layup
angles =[0 90 0 90];
omega_bar=6.5455; % the fundamental frequency multiplier

% More anisotropy:
% L_t =5;
% E1_E2 = 40;% this ratio is variable
% % Anti-symmetric layup
% angles =[0 90 0 90];
% omega_bar=10.6798; % the fundamental frequency multiplier

% Increased number of layers:
% L_t =5;
% E1_E2 = 40;% this ratio is variable
% % Anti-symmetric layup
% angles =[0 90 0 90  0 90 0 90 0 90];
% omega_bar=11.6245; % the fundamental frequency multiplier

% Compute  all the other parameters
E1=181000; E2=E1/E1_E2; E3=E2; G12=0.6*E2;  G13=0.6*E2; G23=0.5*E2;% MPa
nu12= 0.25; nu13= 0.25; nu23= 0.25; rho =1.5e-9;
L=200; W=200; t=L/L_t;% mm

% Fundamental frequency
omega_fundamental =omega_bar*t/W^2*sqrt(E2/rho);
f_fundamental =omega_fundamental/2/pi

nLayers =length(angles);
[nL,nW] =adeal(1*[3,3]);
nts= 1*ones(length(angles),1);% number of elements per layer
ts= t/length(angles)*ones(length(angles),1);% layer thicknesses
graphics = ~false;

    function Rm = LayerRm(XYZ, ts, label)% label equals the layer number here
        Rm= rotmat(angles(label)/180*pi* [0,0,1]);
    end
% Mesh
[fens,gcells] = H8_composite_plate(L,W,ts,nL,nW,nts);;
[fens,gcells] = H8_to_H64(fens,gcells);
% gv=drawmesh( {fens,subset(gcells,gcell_select(fens,gcells,struct('label', 1)))},'gcells', 'facecolor','r');
% gv=drawmesh( {fens,subset(gcells,gcell_select(fens,gcells,struct('label', 2)))},'gv',gv,'gcells', 'facecolor','g');
% gv=drawmesh( {fens,subset(gcells,gcell_select(fens,gcells,struct('label', 3)))},'gv',gv,'gcells', 'facecolor','b');
% gv=drawmesh( {fens,subset(gcells,gcell_select(fens,gcells,struct('label', 4)))},'gv',gv,'gcells', 'facecolor','m');
% Material
prop = property_linel_ortho (...
    struct('E1',E1,'E2',E2,'E3',E3,'G12',G12,'G13',G13,'G23',G23,'nu12',nu12,'nu13',nu13,'nu23',nu23,'rho',rho)); %% 1627 kg/m^3
mater = mater_defor_ss_linel_triax (struct('property',prop));
% Finite element block
feb = feblock_defor_ss (struct ('mater',mater,'gcells',gcells, ...
    'integration_rule',gauss_rule (3,4),'Rm',@LayerRm));

OmegaShift=(2*pi*40) ^ 2;

% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
limits =reshape([min(get(geom,'values'));max(get(geom,'values'))], 6, 1);
% Define the displacement field
u   = 0*geom; % zero out

%  Simple  support EBC
nl0=[fenode_select(fens, struct('box', [0,0,-Inf,Inf,-Inf,Inf],...
    'inflate',0.001*t))];
nl1=[fenode_select(fens, struct('box', [L,L,-Inf,Inf,-Inf,Inf],...
    'inflate',0.001*t))];
nw0=[fenode_select(fens, struct('box', [0,L,0,0,-Inf,Inf],...
    'inflate',0.001*t))];
nw1=[fenode_select(fens, struct('box', [0,L,W,W,-Inf,Inf],...
    'inflate',0.001*t))];

% Suppress transverse displacements
u   = set_ebc(u,nl0,nl0*0+1, [3], nl0*0);
u   = set_ebc(u,nl1,nl1*0+1, [3], nl1*0);
u   = set_ebc(u,nw0,nw0*0+1, [3], nw0*0);
u   = set_ebc(u,nw1,nw1*0+1, [3], nw1*0);

% Suppress tangential displacements along the edges
u   = set_ebc(u,nl0,nl0*0+1, [2], nl0*0);
u   = set_ebc(u,nl1,nl1*0+1, [2], nl1*0);
u   = set_ebc(u,nw0,nw0*0+1, [1], nw0*0);
u   = set_ebc(u,nw1,nw1*0+1, [1], nw1*0);

u   = apply_ebc (u);
% Number equations
u   = numbereqns (u);

% Assemble the system matrix
K = start (sparse_sysmat, get(u, 'neqns'));
evs=stiffness(feb, geom, u);
K = assemble (K, evs);
M = start (sparse_sysmat, get(u, 'neqns'));
ems=mass(feb, geom, u);
% If lumped mass matrix, uncomment this block
% mat=get(ems,'mat');
% for j=1:length(mat)% Hinton, Rock, Zienkiewicz lumping
%     Me=mat{j};
%     em2=sum(sum(Me));
%     dem2=sum(diag(Me));
%     mat{j}=diag(diag(Me)/dem2*em2);
% end
% ems=set(ems,'mat',mat);
% % Consistent or lumped mass matrix
M = assemble (M, ems);

disp(['Number of DOF =' num2str(get(u, 'neqns'))]);

%
neigvs = 10;
options.issym= true;
options.tol=1e-35;
options.maxit= 500;
options.disp= 0;
[W,Omega]=eigs(get(K,'mat')+OmegaShift*get(M,'mat'),get(M,'mat'),neigvs,'SM', options);
Omega =diag(diag(Omega)-OmegaShift);
[Ignore,ix] =sort (diag(Omega));



for i=1:neigvs
    disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) ' [Hz]']);
end


% Plot
if graphics
    Camera =1.0e+003 * [ -0.5986   -1.2414    0.8919    0.0992    0.0992    0.0193         0         0    0.0010    0.0068];
    xyz=get(geom,'values');
    box =reshape([min(xyz);max(xyz)],1,6);
    annloc=[mean(box(1:2)), mean(box(3:4))-mean(box(1:2)), mean(box(5:6))-1.2*t];
    bg=mesh_bdry(gcells);
    bfeb = feblock(struct ('gcells',bg));
    gv=graphic_viewer;
    gv=reset (gv,struct('limits',  limits));
    scale=365;
    w=clone(u,'w'); % make a copy of u
    
    for i=1:2
        f=sqrt(Omega(ix(i),ix(i)))/2/pi;
        disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(f) ' [Hz]']);
        %         clf;
        w = scatter_sysvec(w, W(:,ix(i)));
        w = w*(1/norm(w ));
        wv = get(w,'values');
        wmag = real (sqrt(wv(:,1).^2+wv(:,2).^2+wv(:,3).^2));
        dcm=data_colormap(struct ('range',[min(wmag),max(wmag)], 'colormap',jet));
        colors=map_data(dcm, wmag);
        colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
        frame=1;
        for xscale=scale*sin((0:1:42)/21*2*pi)
            gv=reset (gv,struct('limits', inflate_box(box,scale*max(wmag))));
                        camset(gv,Camera);
            draw(bfeb,gv, struct ('x', geom, 'u', xscale*w,'colorfield',colorfield));
            draw_text(gv, annloc,['  Eigenvector ' num2str(i) '\newline     ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) ' [Hz]'],...
                struct('fontsize', 18,'backgroundcolor','w'));
            draw_axes (gv,struct('length',20 ));
            pause(0.005);
            gif_animation_add_frame(gcf,frame,[mfilename '-m-' num2str(i) '.gif'],30)
            frame=frame+1;
        end
        %     saveas(gcf, ['twist_t4-' num2str(i) '.png'], 'png');
        pause(2); % next eigenvector
        gv=reset (gv,[]);
        %         camset(gv,Camera);
        draw(bfeb,gv, struct ('x', geom,'u',+scale*w,'colorfield',colorfield));
        draw(bfeb,gv, struct ('x', geom,'u',0*w,'facecolor','yellow','edgecolor',.53+[0, 0, 0],'alpha', 0.2));
        draw_text(gv, annloc,['  Eigenvector ' num2str(i) '\newline     ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) '  [Hz]'],...
            struct('fontsize', 18));
        draw_axes (gv,struct('length',20 ));
        pause(2.55);
        
    end
end

end