% Three-dimensional Elasticity Solution for Uniformly Loaded Cross-ply
% Laminates and Sandwich Plates
% Ashraf M. Zenkour, Journal of Sandwich Structures and Materials 2007 9: 213-238
% DOI: 10.1177/1099636207065675
%
function Z_laminate_u_ss
u= physical_units_struct;
% E1=172350; E2=6894; E3=6894; G12=3447;  G13=3447; G23=1378.8;
E1=25*u.MEGA*u.PSI; E2=1*u.MEGA*u.PSI; E3=E2; G12=0.5*u.MEGA*u.PSI;  G13=G12; G23=0.2*u.MEGA*u.PSI;
nu12= 0.25; nu13= 0.25; nu23= 0.25;
a=200*u.MM; b=600*u.MM; 
q0 = -1*u.PSI;
% The below values come from Table 2
h=a/4; wc_analytical  =3.65511/(100*E3*h^3/a^4/q0);
% h=a/10; wc_analytical  =1.16899/(100*E3*h^3/a^4/q0);
h=a/50; wc_analytical  =0.66675/(100*E3*h^3/a^4/q0);
h=a/100; wc_analytical  =0.65071/(100*E3*h^3/a^4/q0);

angles =[0,90,0];
nLayers =length(angles);
[na,nb] =adeal(8*[4,4]);
nts= 1*ones(nLayers,1);% number of elements per layer
ts= h/nLayers*ones(nLayers,1);% layer thicknesses
tolerance =min(ts)/max(nts)/100;
graphics = false;

    function Rm = LayerRm(XYZ, ts, label)% label equals the layer number here
        Rm= rotmat(angles(label)/180*pi* [0,0,1]);
    end
% Mesh
[fens,gcells] = H8_composite_plate(a,b,ts,na,nb,nts);;
[fens,gcells] = H8_to_H20(fens,gcells);
% gv=drawmesh( {fens,subset(gcells,gcell_select(fens,gcells,struct('label', 1)))},'gcells', 'facecolor','r');
% gv=drawmesh( {fens,subset(gcells,gcell_select(fens,gcells,struct('label', 2)))},'gv',gv,'gcells', 'facecolor','g');
% gv=drawmesh( {fens,subset(gcells,gcell_select(fens,gcells,struct('label', 3)))},'gv',gv,'gcells', 'facecolor','b');
% gv=drawmesh( {fens,subset(gcells,gcell_select(fens,gcells,struct('label', 4)))},'gv',gv,'gcells', 'facecolor','m');
% Material
prop = property_linel_ortho (...
    struct('E1',E1,'E2',E2,'E3',E3,'G12',G12,'G13',G13,'G23',G23,'nu12',nu12,'nu13',nu13,'nu23',nu23));
mater = mater_defor_ss_linel_triax (struct('property',prop));
% Finite element block
feb = feblock_defor_ss (struct ('mater',mater,'gcells',gcells, ...
    'integration_rule',gauss_rule (3,2),'Rm',@LayerRm));

% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
limits =reshape([min(get(geom,'values'));max(get(geom,'values'))], 6, 1);
% Define the displacement field
u   = 0*geom; % zero out

%  Simple  support EBC
nl0=[fenode_select(fens, struct('box', [0,0,-Inf,Inf,-Inf,Inf],...
    'inflate',tolerance))];
nl1=[fenode_select(fens, struct('box', [a,a,-Inf,Inf,-Inf,Inf],...
    'inflate',tolerance))];
nw0=[fenode_select(fens, struct('box', [0,a,0,0,-Inf,Inf],...
    'inflate',tolerance))];
nw1=[fenode_select(fens, struct('box', [0,a,b,b,-Inf,Inf],...
    'inflate',tolerance))];

% Suppress transverse displacements
u   = set_ebc(u,nl0,nl0*0+1, [3], nl0*0);
u   = set_ebc(u,nl1,nl1*0+1, [3], nl1*0);
u   = set_ebc(u,nw0,nw0*0+1, [3], nw0*0);
u   = set_ebc(u,nw1,nw1*0+1, [3], nw1*0);

% Suppress tangential displacements along the edges
u   = set_ebc(u,nl0,nl0*0+1, [2], nl0*0);
u   = set_ebc(u,nl1,nl1*0+1, [2], nl1*0);
u   = set_ebc(u,nw0,nw0*0+1, [1], nw0*0);
u   = set_ebc(u,nw1,nw1*0+1, [1], nw1*0);

u   = apply_ebc (u);
% Number equations
u   = numbereqns (u);

% Assemble the system matrix
K = start (sparse_sysmat, get(u, 'neqns'));
evs=stiffness(feb, geom, u);
K = assemble (K, evs);


disp(['Number of DOF =' num2str(get(u, 'neqns'))]);

fi= force_intensity(struct ('magn',[0;0; q0]));
bdry_gcells = mesh_bdry(gcells, []);
bcl = gcell_select(fens, bdry_gcells, ...
    struct ('box',[-Inf,Inf,-Inf,Inf,h,h],'inflate',tolerance));
lfeb = feblock_defor_ss(struct ('mater',mater, 'gcells',subset(bdry_gcells,bcl),...
    'integration_rule',gauss_rule(2,2)));
F = start (sysvec, get(u, 'neqns'));
F = assemble (F, distrib_loads(lfeb, geom, u, fi,2));
u = scatter_sysvec(u, K\F);


% Plot
if graphics
    gv=graphic_viewer;
    gv=reset (gv,[]);
    scale=10;
    % draw(feb,gv, struct ('x', geom,'u',0*u, 'facecolor','none'));
    fld = field_from_integration_points(feb, geom, u, [], 'Cauchy', 1);
    nvals=get(fld,'values');
    dcm=data_colormap(struct ('range',[min(nvals),max(nvals)], 'colormap',jet));
    colorfield=field(struct ('name', ['colorfield'], 'data',map_data(dcm, nvals)));
    draw(feb, gv, struct ('x',geom,'u', scale*u, 'colorfield',colorfield, 'shrink',1));
    colormap(jet);
    cbh=colorbar;
    set(cbh,...
        'Position',[0.8 0.15 0.02 0.7],...
        'YLim',[0,1],...
        'YTick',[0,1],...
        'YTickLabel',{[num2str(min(nvals))],[num2str(max(nvals))]});%{[num2str((min(nvals)))],[num2str((max(nvals)))]}
    set(get(cbh,'XLabel'),'String','\sigma_{y}');
    title ('Stress');
    %         view3d rot
    view([0,0])
    %         camset (gv, [362.6179  179.7380  186.8838    4.5277   24.0360   -3.5642   -0.4020   -0.1748  0.8988    3.9476])
    %                 camset (gv, [-577.5531  612.4095  322.0260    4.5854   46.3526  -10.6875    0.2718   -0.2643    0.9253    3.5144])
    set(gcf, 'Position', get(0,'Screensize')); % Maximize figure.
end
nc=[fenode_select(fens, struct('box', [a/2,a/2,b/2,b/2,-Inf,Inf],...
    'inflate',tolerance))];
displacement=gather(u,nc,'values','noreshape');
wc =mean(displacement (:,3) );
wc/wc_analytical
end