% Fuse two node sets. If necessary, by gluing together nodes located within tolerance of each other.
%
% [fens,fens1,fens2] = fuse_nodes(fens1, fens2, tolerance)
%
% The two node sets, fens1 and fens2,  are fused together by
% merging the nodes that fall within a box of size "tolerance".
% The merged node set, fens, and the two sets with
% renumbered  connectivities are returned.
% 
% The IDs of fens1,fens2 will point to the new nodes in the node set fens.
% The node set IDs fens1 will have changed, whereas the IDs of the node set
% fens2 all are guaranteed to be the same.
% 
% Example: After the call to this function we have
% id1 = get(fens1,'id');
% id1(j) is the node in the node set fens which used to be node j in node set fens1.
function [fens,fens1,fens2] = fuse_nodes(fens1, fens2, tolerance)
    xyz1 = get(fens1,'xyz');
    dim =size(xyz1,2); 
    id1 = get(fens1,'id');
    xyz2 = get(fens2,'xyz');
    id2 = get(fens2,'id');
    c1=ones(size(xyz2,1),1);
    % Mark nodes from the first array that are duplicated in the second
    if (tolerance>0)% should we attempt to merge nodes?
        for i=1:count(fens1)
            xyz =xyz1(i,:);
            xyzd=abs(xyz2-c1*xyz);
            j=find(sum(xyzd')'<tolerance);
            if (~isempty(j))
                id1(i) =-j(1);
            end
        end
    end
    % Generate  fused arrays of the nodes
    xyzm = zeros(count(fens1)+count(fens2),dim);
    xyzm(1:count(fens2),1:size(xyz2,2))=xyz2;% fens2 are there without change
    idm = zeros(count(fens1)+count(fens2),1);
    idm(1:count(fens2))=(1:count(fens2));
    mid=count(fens2)+1;
    for i=1:count(fens1) % and then we pick only non-duplicated fens1
        if id1(i)>0
            id1(i)=mid;
            idm(mid)=mid;
            xyzm(mid,:)=xyz1(i,:);
            mid=mid+1;
        else 
            id1(i)=id2(-id1(i));
        end
    end
    nfens =mid-1;
    xyzm =xyzm(1:nfens,:);
    idm =idm(1:nfens,:);
    
    % Create the fused Node set
    fens =fenodeset(struct('xyz',xyzm,'id',idm));
    % The Node set 1 numbering will change
    fens1=set(fens1,'id',id1);
    % The node set 2 numbering stays the same
end
